;
; File:
;                         floppy.asm
; Description:
;                   floppy disk driver primitives
;
;                       Copyright (c) 1995
;                       Pasquale J. Villani
;                       All Rights Reserved
;
; This file is part of DOS-C.
;
; DOS-C is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version
; 2, or (at your option) any later version.
;
; DOS-C is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
; the GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public
; License along with DOS-C; see the file COPYING.  If not,
; write to the Free Software Foundation, 675 Mass Ave,
; Cambridge, MA 02139, USA.
;
; $Id: floppy.asm,v 1.13 2002/12/09 00:17:08 bartoldeman Exp $
;

%define PC98HDD

%ifndef SYS
     %include "..\kernel\segs.inc"
     segment _TEXT
%else
     segment _TEXT class=CODE
%endif
;
;
; Reset both the diskette and hard disk system
;
; BOOL fl_reset(WORD drive)
;
;       returns TRUE if successful
;

		global	_fl_reset
_fl_reset:
%ifdef PC98HDD
		mov	bx, sp
		mov	ah, 03h		; BIOS initialize
		mov	al, [bx + 2]	; DA/UA
		int	1bh

		jc	.err
		mov	ax, 1		; TRUE on success
		ret

	.err:
		xor	ax, ax		; FALSE on error
		ret
%else
		xor	ax, ax		; FALSE on error
		ret
%endif


;
;
; Read DASD Type
;
; COUNT fl_readdasd(WORD drive)
;
;       returns 0-3 if successful, 0xFF if error
;
; Code   Meaning
;  0     The drive is not present
;  1     Drive present, cannot detect disk change
;  2     Drive present, can detect disk change
;  3     Fixed disk
;
%if 0
		global	_fl_readdasd
_fl_readdasd:
%endif

;
;
; Read disk change line status
;
; COUNT fl_diskchanged(WORD drive)
;
;       returns 1 if disk has changed, 0 if not, 0xFF if error
;

		global	_fl_diskchanged
_fl_diskchanged:
%ifdef PC98HDD
		xor	ax, ax		; disk has not changed
		ret
%else
		mov	ax, 0ffh	; 0xFF on error
		ret
%endif


;
; Read the disk system status
;
; COUNT fl_rd_status(WORD drive)
;
; Returns error codes
;
; See Phoenix Bios Book for error code meanings
;
%if 0
		global	_fl_rd_status
_fl_rd_status:
%endif

;
; Format Sectors
;
; COUNT fl_format(WORD drive, WORD head, WORD track, WORD sector, WORD count, BYTE FAR *buffer);
;
; Formats one or more tracks, sector should be 0.
;
; Returns 0 if successful, error code otherwise.
		global	_fl_format
_fl_format:
%ifdef PC98HDD
		xor	ax, ax		; success
		ret
%else
		mov	ax, 40h		; error (Equipment Check)
		ret
%endif

;
; Read Sectors
;
; COUNT fl_read(WORD drive, WORD head, WORD track, WORD sector, WORD count, BYTE FAR *buffer);
;
; Reads one or more sectors.
;
; Returns 0 if successful, error code otherwise.
;
;
; Write Sectors
;
; COUNT fl_write(WORD drive, WORD head, WORD track, WORD sector, WORD count, BYTE FAR *buffer);
;
; Writes one or more sectors.
;
; Returns 0 if successful, error code otherwise.
;
		global	_fl_read
_fl_read:
		mov	ah, 06h		; cmd READ
		jmp	short fl_common

		global	_fl_verify
_fl_verify:
		mov	ah, 01h		; cmd verify
		jmp	short fl_common

		global	_fl_write
_fl_write:
		mov	ah, 05h		; cmd WRITE
%if 0
		xor	ax, ax
		ret
%endif

fl_common:
%ifdef PC98HDD
		push	bp		; C entry
		mov	bp, sp
		push	es

		mov	al, [bp + 4]	; DA/UA
		or	al, 80h
		mov	bx, [bp + 0ch]	; count to read/write
		mov	cl, 9
		shl	bx, cl		; * 512
		mov	cx, [bp + 8]	; cylinder number
		mov	dh, [bp + 6]	; head number
		mov	dl, [bp + 0ah]	; sector number
		dec	dl
		les	bp, [bp + 0eh]	; Load 32 bit buffer ptr

		int	1bh
		mov	al, ah
		xor	ah, ah

		pop	es
		pop	bp
		ret
%else
		mov	ax, 40h		; error (Equipment Check)
		ret
%endif


%if 0
;
;
; Get number of disks
;
; COUNT fl_nrdrives(VOID)
;
;       returns AX = number of harddisks
;

		global	_fl_nrdrives
_fl_nrdrives:
%endif

; ---------------------------------------------------------------------------
;                             LBA Specific Code
;
;
; Added by:  Brian E. Reifsnyder
; ---------------------------------------------------------------------------

;
;
; Check for Enhanced Disk Drive support (EDD).  If EDD is supported then
;       LBA can be utilized to access the hard disk.
;
; unsigned int lba_support(WORD hard_disk_number)
;
;       returns TRUE if LBA can be utilized
;
%if 0
		global  _fl_lba_support
_fl_lba_support:

;
; Read Sectors
;
; COUNT fl_lba_read(WORD drive, WORD dap_segment, WORD dap_offset);
;
; Reads one or more sectors.
;
; Returns 0 if successful, error code otherwise.
;
;
; Write Sectors
;
; COUNT fl_lba_write(WORD drive, WORD dap_segment, WORD dap_offset);
;
; Writes one or more sectors.
;
; Returns 0 if successful, error code otherwise.
;
		global  _fl_lba_read
_fl_lba_read:
		global  _fl_lba_write
_fl_lba_write:
		global  _fl_lba_write_and_verify
_fl_lba_write_and_verify:
		global  _fl_lba_verify
_fl_lba_verify:
%endif

; COUNT fl_lba_ReadWrite(BYTE drive, UWORD mode, VOID FAR *dap_p)
;
; Returns 0 if successful, error code otherwise.
;
		global  _fl_lba_ReadWrite
_fl_lba_ReadWrite:
		mov	ax, 40h		; error (Equipment Check)
		ret

global _fl_readkey
_fl_readkey:
		xor	ah, ah
		int	18h
		ret

global _fl_setdisktype
_fl_setdisktype:
		mov	ax, 40h		; error (Equipment Check)
		ret

global _fl_setmediatype
_fl_setmediatype:
		mov	ax, 40h		; error (Equipment Check)
		ret

