/****************************************************************/
/*                                                              */
/*                           sysclk.c                           */
/*                                                              */
/*                     System Clock Driver                      */
/*                                                              */
/*                      Copyright (c) 1995                      */
/*                      Pasquale J. Villani                     */
/*                      All Rights Reserved                     */
/*                                                              */
/* This file is part of DOS-C.                                  */
/*                                                              */
/* DOS-C is free software; you can redistribute it and/or       */
/* modify it under the terms of the GNU General Public License  */
/* as published by the Free Software Foundation; either version */
/* 2, or (at your option) any later version.                    */
/*                                                              */
/* DOS-C is distributed in the hope that it will be useful, but */
/* WITHOUT ANY WARRANTY; without even the implied warranty of   */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See    */
/* the GNU General Public License for more details.             */
/*                                                              */
/* You should have received a copy of the GNU General Public    */
/* License along with DOS-C; see the file COPYING.  If not,     */
/* write to the Free Software Foundation, 675 Mass Ave,         */
/* Cambridge, MA 02139, USA.                                    */
/****************************************************************/

#include "portab.h"
#include "globals.h"

#ifdef VERSION_STRINGS
static char *RcsId =
    "$Id: sysclk.c,v 1.17 2002/12/09 00:17:15 bartoldeman Exp $";
#endif

/*                                                                      */
/* WARNING - THIS DRIVER IS NON-PORTABLE!!!!                            */
/*                                                                      */

typedef struct {
	BYTE	year;
	struct {
		BITS	mday:4;
		BITS	month:4;
	} _;
	BYTE	day;
	BYTE	hour;
	BYTE	minute;
	BYTE	second;
} CAL_DATA;

STATIC int ByteToBcd(int x)
{
  return ((x / 10) << 4) | (x % 10);
}
STATIC int BcdToByte(int x)
{
  return ((x >> 4) & 0xf) * 10 + (x & 0xf);
}

WORD ASMCFUNC FAR clk_driver(rqptr rp)
{
  switch (rp->r_command)
  {
    case C_INIT:
      /* rp->r_endaddr = device_end(); not needed - bart */
      rp->r_nunits = 0;
      return S_DONE;

    case C_INPUT:
      {
        struct ClockRecord clk;
        CAL_DATA cal;
        unsigned Year;

        Read98Clock(&cal);

        Year = BcdToByte(cal.year);
        if (Year >= 80)
          Year += 1900;
        else
          Year += 2000;

        clk.clkDays			= DaysFromYearMonthDay(Year, cal._.month, BcdToByte(cal.day));
        clk.clkMinutes		= BcdToByte(cal.minute);
        clk.clkHours		= BcdToByte(cal.hour);
        clk.clkHundredths	= 0;
        clk.clkSeconds		= BcdToByte(cal.second);;

        fmemcpy(rp->r_trans, &clk,
                min(sizeof(struct ClockRecord), rp->r_count));
      }
      return S_DONE;

    case C_OUTPUT:
      {
        const unsigned short *pdays;
        unsigned Day, Month, Year;
        struct ClockRecord clk;
        CAL_DATA cal;
        
        rp->r_count = min(rp->r_count, sizeof(struct ClockRecord));
        fmemcpy(&clk, rp->r_trans, rp->r_count);

        /* Fix year by looping through each year, subtracting   */
        /* the appropriate number of days for that year.        */
        for (Year = 1980, Day = clk.clkDays;;)
        {
          pdays = is_leap_year_monthdays(Year);
          if (Day >= pdays[12])
          {
            ++Year;
            Day -= pdays[12];
          }
          else
            break;
        }
            
        /* Day contains the days left and count the number of   */
        /* days for that year.  Use this to index the table.    */
        for (Month = 1; Month < 13; ++Month)
        {
          if (pdays[Month] > Day)
          {
            Day = Day - pdays[Month - 1] + 1;
            break;
          }
        }

        cal.year	= ByteToBcd(Year % 100);
        cal._.mday	= 0;
        cal._.month	= ByteToBcd(Month);
        cal.day		= ByteToBcd(Day);
        cal.hour	= ByteToBcd(clk.clkHours);
        cal.minute	= ByteToBcd(clk.clkMinutes);
        cal.second	= ByteToBcd(clk.clkSeconds);

        Write98Clock(&cal);
      }
      return S_DONE;

    case C_OFLUSH:
    case C_IFLUSH:
      return S_DONE;

    case C_OUB:
    case C_NDREAD:
    case C_OSTAT:
    case C_ISTAT:
    default:
      return failure(E_FAILURE);        /* general failure */
  }
}

